#include "i3d/image3d.h"
#include "similarityMeasures.h"


template <typename VOXEL>
float ComputeSAVD(i3d::Image3d<VOXEL> const &A,
					 i3d::Image3d<VOXEL> const &B)
{
	if (A.GetSize() != B.GetSize())
		throw i3d::LibException("Input images don't have the same size.");

	//porovnani, zda jsou oba obrazy obsahove stejne
	const VOXEL* a=A.GetFirstVoxelAddr();
	const VOXEL* aS=a+A.GetImageSize();
	const VOXEL* b=B.GetFirstVoxelAddr();
	double sum=0;
	while (a != aS) {
		sum+=abs( (double)*a - (double)*b );
		++a; ++b;
	}

	//NORMALIZACE, ZKUSTE SCHVALNE I BEZ NI
	sum/=(double)A.GetImageSize();

	//std::cout << "suma abs. rozdilu obrazu je " << sum << std::endl;

	return((float)sum);
}


template <typename VOXEL>
float ComputeSAVD(i3d::Image3d<VOXEL> const &A,
					 i3d::VOI<i3d::PIXELS> const & voiA,
					 i3d::Image3d<VOXEL> const &B,
					 i3d::VOI<i3d::PIXELS> const & voiB)
{
	if (voiA.size != voiB.size)
		throw i3d::InternalException("Input VOIs don't have the same size.");

	if ((!A.Include(voiA.offset)) || (!A.Include(voiA.offset+voiA.size-i3d::Vector3d<int>(1,1,1))))
		throw i3d::InternalException("VOI for image 'A' does not lie entirely within the image.");
	if ((!B.Include(voiB.offset)) || (!B.Include(voiB.offset+voiB.size-i3d::Vector3d<int>(1,1,1))))
		throw i3d::InternalException("VOI for image 'B' does not lie entirely within the image.");

	double sum=0;

	for (int z=0; z < (int)voiA.size.z; ++z)
	  for (int y=0; y < (int)voiA.size.y; ++y)
	    for (int x=0; x < (int)voiA.size.x; ++x) {
			i3d::Vector3d<int> pos(x,y,z);
			sum+=abs( (double)A.GetVoxel(voiA.offset+pos)
					  - (double)B.GetVoxel(voiB.offset+pos) );
		 }
	//std::cout << "suma abs. rozdilu VOI je " << sum << std::endl;

	//NORMALIZACE, ZKUSTE SCHVALNE I BEZ NI
	sum/=(double)voiA.size.x * (double)voiA.size.y;

	return((float)sum);
}


template float ComputeSAVD(i3d::Image3d<i3d::GRAY8> const &A, i3d::Image3d<i3d::GRAY8> const &B);
template float ComputeSAVD(i3d::Image3d<float> const &A, i3d::Image3d<float> const &B);

template float ComputeSAVD(i3d::Image3d<i3d::GRAY8> const &A, i3d::VOI<i3d::PIXELS> const & voiA,
					 i3d::Image3d<i3d::GRAY8> const &B, i3d::VOI<i3d::PIXELS> const & voiB);
template float ComputeSAVD(i3d::Image3d<float> const &A, i3d::VOI<i3d::PIXELS> const & voiA,
					 i3d::Image3d<float> const &B, i3d::VOI<i3d::PIXELS> const & voiB);
